unit MapboxGL;

{
  Pascal interface to Mapbox GL JavaScript library.

  Mapbox GL JS API:  https://www.mapbox.com/mapbox-gl-js/api

  Author:    Phil Hess.
  Copyright: Copyright (C) Phil Hess.
  License:   Modified LGPL (see Free Pascal's rtl/COPYING.FPC).
}

interface

{$modeswitch externalclass}

uses
  JS,
  Web;

type
  TLngLatBounds = class;  //forward declaration

  TLngLat = class external name 'mapboxgl.LngLat'
    lng : Double;
    lat : Double;
    constructor New(lng : Double;
                    lat : Double);
    function wrap : TLngLat;
    function toArray : array of Double;
    function toString : string;
    function toBounds(radius : Double) : TLngLatBounds;
    function convert(input : array of Double) : TLngLat;
  end;

  TLngLatLike = class external name 'mapboxgl.LngLat' (TLngLat);  //can also be array of two numbers or object with lng,lat


  TLngLatBounds = class external name 'mapboxgl.LngLatBounds'
    constructor New(sw : TLngLatLike;
                    ne : TLngLatLike);
    function setNorthEast(ne : TLngLatLike) : TLngLatBounds;
    function setSouthWest(sw : TLngLatLike) : TLngLatBounds;
    function getCenter : TLngLat;
    function getSouthWest : TLngLat;
    function getNorthEast : TLngLat;
    function getNorthWest : TLngLat;
    function getSouthEast : TLngLat;
    function getWest : Double;
    function getSouth : Double;
    function getEast : Double;
    function getNorth : Double;
  end;

  TLngLatBoundsLike = class external name 'mapboxgl.LngLatBounds' (TLngLatBounds);  //can also be array of LngLatLike objects


  TPoint = class external name 'mapboxgl.Point'
    x : Double;
    y : Double;
    constructor New(x : Double;
                    y : Double);
     //has other methods too
  end;

  TPointLike = class external name 'mapboxgl.Point' (TPoint);  //can also be array of two numbers


  TSupportedFunc = function(options : TJSObject = nil) : Boolean;


  TMap = class;  //forward declaration

  IControl = class external name 'IControl'  //specification, not really a class
    constructor New();
    function onAdd(map : TMap) : TJSHTMLElement; abstract; virtual;
    procedure onRemove(map : TMap = nil); abstract; virtual;
  end;


  TMapEvent = class external name 'MapMouseEvent'  //not really a class; could be either MapMouseEvent or MapTouchEvent
    &type : string;
    target : TMap;
    originalEvent : TJSEvent;
    point : TPoint;  //pixel coords
    lngLat : TLngLat;  //geo coords
  end;

  TMapListenerProc = procedure(e : TMapEvent = nil);  //e can be omitted?


  TMap = class external name 'mapboxgl.Map'
    constructor New(options : TJSObject);
    function addControl(control  : IControl;
                        position : string = 'top-right') : TMap;
    function removeControl(control : IControl) : TMap;
    function resize : TMap;
    function getBounds : TLngLatBounds;
    function getMaxBounds : TLngLatBounds;
    function setMaxBounds(lnglatbounds : TLngLatBoundsLike = nil) : TMap;
    function setMinZoom(minZoom : Double = 0) : TMap;
    function getMinZoom : Double;
    function setMaxZoom(minZoom : Double = 22) : TMap;
    function getMaxZoom : Double;
    function project(lnglat : TLngLatLike) : TPoint;
    function unproject(point : TPointLike) : TLngLat;
    function on(&type    : string;
                listener : TMapListenerProc) : TMap; overload;
    function on(&type    : string;
                layer    : string;
                listener : TMapListenerProc) : TMap; overload;
    function off(&type    : string;
                 listener : TMapListenerProc) : TMap; overload;
    function off(&type    : string;
                 layer    : string;
                 listener : TMapListenerProc) : TMap; overload;
    function setStyle(style   : TJSObject;
                      options : TJSObject = nil) : TMap; overload;
    function setStyle(style   : string;
                      options : TJSObject = nil) : TMap; overload;
    function getStyle : TJSObject;
    function addSource(id     : string;
                       source : TJSObject) : TMap;
    function removeSource(id : string) : TMap;
    function getSource(id : string) : TJSObject;
    function isStyleLoaded : Boolean;
    function isSourceLoaded(id : string) : Boolean;
    function areTilesLoaded : Boolean;
    function addLayer(layer  : TJSObject;
                      before : string = '') : TMap;
    function moveLayer(layer    : string;
                       beforeId : string = '') : TMap;
    function removeLayer(id : string) : TMap;
    function getLayer(id : string) : TJSObject;
    function getContainer : TJSHTMLElement;
    function getCanvasContainer : TJSHTMLElement;
    function getCanvas : TJSHTMLCanvasElement;
    function loaded : Boolean;
    procedure remove;
    function getCenter : TLngLat;
    function setCenter(center : TLngLatLike) : TMap;
    function panBy(offset    : array of Double;
                   options   : TJSObject = nil;
                   eventData : TJSObject = nil) : TMap; overload;
    function panBy(lnglat    : TLngLatLike;
                   options   : TJSObject = nil;
                   eventData : TJSObject = nil) : TMap; overload;
    function getZoom : Double;
    function setZoom(zoom      : Double;
                     eventData : TJSObject = nil) : TMap;
    function zoomTo(zoom      : Double;
                    options   : TJSObject = nil;
                    eventData : TJSObject = nil) : TMap;
    function zoomIn(options   : TJSObject = nil;
                    eventData : TJSObject = nil) : TMap;
    function zoomOut(options   : TJSObject = nil;
                     eventData : TJSObject = nil) : TMap;
    function getBearing : Double;
    function setBearing(bearing   : Double;
                        eventData : TJSObject = nil) : TMap;
    function rotateTo(bearing   : Double;
                      options   : TJSObject = nil;
                      eventData : TJSObject = nil) : TMap;
    function resetNorth(options   : TJSObject = nil;
                        eventData : TJSObject = nil) : TMap;
    function snapToNorth(options   : TJSObject = nil;
                         eventData : TJSObject = nil) : TMap;
    function getPitch : Double;
    function setPitch(pitch     : Double;
                      eventData : TJSObject = nil) : TMap;
    function fitBounds(bounds    : TLngLatBoundsLike;
                       options   : TJSObject = nil;  //??
                       eventData : TJSObject = nil) : TMap;
    function jumpTo(options   : TJSObject;
                    eventData : TJSObject = nil) : TMap;
    function easeTo(options   : TJSObject;  //also accepts AnimationOptions
                    eventData : TJSObject = nil) : TMap;
    function flyTo(options   : TJSObject;  //also accepts AnimationOptions and others
                   eventData : TJSObject = nil) : TMap;
    function isMoving : Boolean;
    function stop : TMap;
  end;


  TPopup = class external name 'mapboxgl.Popup'
    constructor New(options : TJSObject = nil);  //options can be omitted in new
    function isOpen : Boolean;
    function getLngLat : TLngLat;
    function setLngLat(lnglat : TLngLat) : TPopup;
    function setText(text : string) : TPopup;
    function setHTML(html : string) : TPopup;
    function addTo(map : TMap) : TPopup;
    function remove : TPopup;
    function setDOMContent(htmlNode : TJSElement) : TPopup;
  end;


  TMarker = class external name 'mapboxgl.Marker'
    constructor New(element : TJSHTMLElement = nil;
                    options : TJSObject = nil);
    function addTo(map : TMap) : TMarker;
    function remove : TMarker;
    function getLngLat : TLngLat;
    function setLngLat(lnglat : TLngLat) : TMarker;
    function setPopup(popup : TPopup = nil) : TMarker;
    function getPopup : TPopup;
    function togglePopup : TMarker;
  end;


  TNavigationControl = class external name 'mapboxgl.NavigationControl'
    constructor New();
  end;

  TScaleControl = class external name 'mapboxgl.ScaleControl'
    constructor New(options : TJSObject);  //can options be omitted in new?
  end;

  TFullscreenControl = class external name 'mapboxgl.FullscreenControl'
    constructor New();
  end;


var
  accessToken : string external name 'mapboxgl.accessToken';

  supported : TSupportedFunc external name 'mapboxgl.supported';

  version : string external name 'mapboxgl.version';

  defaultMinZoom : Double external name 'mapboxgl.defaultMinZoom';
  defaultMaxZoom : Double external name 'mapboxgl.defaultMaxZoom';

//  defaultOptions : TJSObject external name 'mapboxgl.defaultOptions';  //but Popup also has defaultOptions


implementation


end.
